import string
from fabric.connection import Connection
from tif.fabric import cli
from tif.files import operations as files
from tif.fabric.logger import Logger
from tif.fabric.CommandPrefix import CommandPrefix

def list_commands(c : Connection, magento_root: string, command_prefix=""):
    with c.cd(magento_root):
        command = "bin/magento list"
        Logger().log("Running command '{}'".format(command))
        c.run(CommandPrefix(command, command_prefix).prefix_command(), pty=True)

def run(c : Connection, magento_root: string, command_prefix=""):
    with c.cd(magento_root):
        list_commands(c, magento_root, command_prefix)
        command = cli.prompt(">>> Enter bin/magento command to run: ")
        command = "bin/magento {}".format(command)
        Logger().log("Running command '{}'".format(command))
        c.run(CommandPrefix(command, command_prefix).prefix_command())

def run_command(c : Connection, magento_root: string, command: string, command_prefix=""):
    with c.cd(magento_root):
        command = "bin/magento {}".format(command)
        Logger().log("Running command '{}'".format(command))
        c.run(CommandPrefix(command, command_prefix).prefix_command())

def install(c : Connection, install_dir : string, command_prefix=""):
    with c.cd(install_dir):
        command = "bin/magento setup:install"
        Logger().log("Running command '{}'".format(command))
        c.run(CommandPrefix(command, command_prefix).prefix_command())

def mode_show(c : Connection, install_dir : string, command_prefix=""):
    with c.cd(install_dir):
        command = "bin/magento deploy:mode:show"
        Logger().log("Running command '{}'".format(command))
        c.run(CommandPrefix(command, command_prefix).prefix_command())

def delete_generated(c : Connection, magento_root : string, command_prefix=""):
    with c.cd(magento_root):
        confirm = cli.cli_confirm("You are about to empty the generated directory, Are you sure?")
        if (confirm == "y"):
            command = "rm -rf generated/*"
            Logger().log("Running command '{}'".format(command))
            c.run(CommandPrefix(command, command_prefix).prefix_command())
            cli.puts(".:~ Done")
        else:
            cli.puts("!!! Aborted")

def delete_static(c : Connection, magento_root : string, command_prefix=""):
    with c.cd(magento_root):
        confirm = cli.cli_confirm("You are about to empty the static directory, Are you sure?")
        if (confirm == "y"):
            command = "rm -rf pub/static/*"
            Logger().log("Running command '{}'".format(command))
            c.run(CommandPrefix(command, command_prefix).prefix_command())
            cli.puts(".:~ Done")
        else:
            cli.puts("!!! Aborted")

def delete_logs(c : Connection, magento_root : string, command_prefix=""):
    with c.cd(magento_root):
        confirm = cli.cli_confirm("You are about to empty the var/log directory, Are you sure?")
        if (confirm == "y"):
            command = "rm -rf var/log/*"
            Logger().log("Running command '{}'".format(command))
            c.run(CommandPrefix(command, command_prefix).prefix_command())
            cli.puts(".:~ Done")
        else:
            cli.puts("!!! Aborted")

def delete_cache(c : Connection, magento_root : string, command_prefix=""):
    with c.cd(magento_root):
        confirm = cli.cli_confirm("You are about to empty the var/cache directory, Are you sure?")
        if (confirm == "y"):
            command = "rm -rf var/cache/*"
            Logger().log("Running command '{}'".format(command))
            c.run(CommandPrefix(command, command_prefix).prefix_command())
            cli.puts(".:~ Done")
        else:
            cli.puts("!!! Aborted")

def di_compile(c : Connection, magento_root : string, command_prefix=""):
    with c.cd(magento_root):
        command = "bin/magento setup:di:compile"
        Logger().log("Running command '{}'".format(command))
        c.run(CommandPrefix(command, command_prefix).prefix_command())

def module_enable(c : Connection, magento_root : string, command_prefix=""):
    with c.cd(magento_root):
        module = cli.prompt(">>> Enter module to enable: ")
        if (module):
            command = "bin/magento module:enable {}".format(module)
            Logger().log("Running command '{}'".format(command))
            c.run(CommandPrefix(command, command_prefix).prefix_command())
        else:
            cli.puts("!!! Aborted")

def setup_upgrade(c : Connection, magento_root : string, command_prefix=""):
    with c.cd(magento_root):
        command = "bin/magento setup:upgrade"
        Logger().log("Running command '{}'".format(command))
        c.run(CommandPrefix(command, command_prefix).prefix_command())

def setup_static_content_deploy(c : Connection, magento_root : string, options : None, command_prefix = ""):
    with c.cd(magento_root):
        command = "bin/magento setup:static-content:deploy"
        if options:
            command = "bin/magento setup:static-content:deploy {}".format(options)
        Logger().log("Running command '{}'".format(command))
        c.run(CommandPrefix(command, command_prefix).prefix_command())

def cache_clean(c : Connection, magento_root : string, command_prefix=""):
    with c.cd(magento_root):
        command = "bin/magento cache:clean"
        Logger().log("Running command '{}'".format(command))
        c.run(CommandPrefix(command, command_prefix).prefix_command())

def cache_flush(c : Connection, magento_root : string, command_prefix=""):
    with c.cd(magento_root):
        command = "bin/magento cache:flush"
        Logger().log("Running command '{}'".format(command))
        c.run(CommandPrefix(command, command_prefix).prefix_command())

def template_hints(c : Connection, magento_root : string, command_prefix=""):
    with c.cd(magento_root):
        status = cli.prompt(">>> Enter disable or enable for template hints: ")
        if (status == "enable" or status == "disable"):
            command = "bin/magento dev:template-hints:{}".format(status)
            Logger().log("Running command '{}'".format(command))
            c.run(CommandPrefix(command, command_prefix).prefix_command())
        else:
            cli.puts("!!! Aborted")

def generate_whitelist(c : Connection, magento_root : string, command_prefix=""):
    with c.cd(magento_root):
        module_name = cli.prompt(">>> Enter module name to generate whitelist json: ")
        command = "bin/magento setup:db-declaration:generate-whitelist --module-name={}".format(module_name)
        Logger().log("Running command '{}'".format(command))
        c.run(CommandPrefix(command, command_prefix).prefix_command())

def run_cron_group(c : Connection, magento_root : string, command_prefix=""):
        with c.cd(magento_root):
            group = cli.prompt(">>> Specify cron group to run: ")
            command = "bin/magento cron:run --group {}".format(group)
            Logger().log("Running command '{}'".format(command))
            c.run(CommandPrefix(command, command_prefix).prefix_command())

def cron_install(c : Connection, magento_root : string, command_prefix=""):
        with c.cd(magento_root):
            command = "bin/magento cron:install"
            Logger().log("Running command '{}'".format(command))
            c.run(CommandPrefix(command, command_prefix).prefix_command())

def run_cron(c : Connection, magento_root : string, command_prefix=""):
        with c.cd(magento_root):
            command = "bin/magento cron:run"
            Logger().log("Running command {}".format(command))
            c.run(CommandPrefix(command, command_prefix).prefix_command())

def cat_log(c : Connection, magento_root : string, command_prefix=""):
        with c.cd(magento_root + "/var/log"):
            command = "ls"
            Logger().log("Running command '{}'".format(command))
            c.run(CommandPrefix(command, command_prefix).prefix_command(), pty=True)
            log = cli.prompt(">>> Specify log to cat: ")
            command = "cat {}".format(log)
            Logger().log("Running command '{}'".format(command))
            c.run(CommandPrefix(command, command_prefix).prefix_command(), pty=True)

def tail_log(c : Connection, magento_root : string, command_prefix=""):
        with c.cd(magento_root + "/var/log"):
            command = "ls"
            Logger().log("Running command '{}'".format(command))
            c.run(CommandPrefix(command, command_prefix).prefix_command(), pty=True)
            log = cli.prompt(">>> Specify log to tail: ")
            files.tail_f(c, magento_root + "/var/log", log, command_prefix)

def admin_user_create(c: Connection, magento_root : string, command_prefix=""):
    with c.cd(magento_root):
        command = "bin/magento admin:user:create"
        Logger().log("Running command '{}'".format(command))
        c.run(CommandPrefix(command, command_prefix).prefix_command(), pty=True)

def enable_maintenance_mode(c : Connection, magento_root : string, command_prefix=""):
    with c.cd(magento_root):
        command = "bin/magento maintenance:enable"
        Logger().log("Running command '{}'".format(command))
        c.run(CommandPrefix(command, command_prefix).prefix_command(), pty=True)

def indexer_reindex(c : Connection, magento_root : string, command_prefix="", indexers = None):
    with c.cd(magento_root):
        if indexers:
            command = "bin/magento indexer:reindex {}".format(indexers)
        else:
            command = "bin/magento indexer:reindex"
        Logger().log("Running command '{}'".format(command))
        c.run(CommandPrefix(command, command_prefix).prefix_command(), pty=True)



import string
import re
import os
from tif.fabric import cli
from fabric.connection import Connection
from tif.docker.service import Service

class Options:
    def __init__(self, **options) -> None:
        self.options = options

    def render(self, input_text= None, confirm_prompt = True) -> string:
        self._print()
        if (input_text):
            option = input(input_text)
        else:
            option = input("Choose an option: ")
        return self._get_option(option.strip(), confirm_prompt=confirm_prompt)

    def docker_container_chooser(self, service : Service, confirm_prompt = True) -> string:
        containers = service.containers_dictionary()
        containers[str(len(containers) + 1)] = "Exit"
        self.options = containers
        return self.render(input_text="Choose a container: ", confirm_prompt=confirm_prompt)


    def remote_directory_chooser(self, c : Connection, dir = None, absolute_path = True, input_text = None, list_all = False):
        options = []
        dir = dir or c.cwd
        dir_list = c.sftp().listdir(dir)
        if not list_all:
            filtered_dir_list = [file for file in dir_list if (not file.startswith(".") and not re.search(r"\.([\w\d]){1,}$", file))]
        else:
            filtered_dir_list = dir_list
        filtered_dir_list.sort()
        for file in filtered_dir_list:
            print (cli.puts_hide(">>> {}".format(filtered_dir_list.index(file) + 1)), '--', file)
        if (input_text):
            option = input(input_text)
        else:
            option = input("Choose an option: ")
        if not option.isnumeric():
            cli.puts("!!! Invalid option selected")
            exit()
        chosen = cli.cli_confirm("You have selected the directory '{}'. Keep this choice?".format(filtered_dir_list[int(option) - 1]))
        if chosen == 'y':
            if not absolute_path:
                return os.sep.join(options) + filtered_dir_list[int(option) - 1]
            else:
                return dir + os.sep + os.sep.join(options) + filtered_dir_list[int(option) - 1]
        else:
            while(option is not None):
                options.append(filtered_dir_list[int(option) - 1])
                c.sftp().chdir(dir + os.sep + os.sep.join(options))
                dir_list = c.sftp().listdir(dir + os.sep + os.sep.join(options))
                if not list_all:
                    filtered_dir_list = [file for file in dir_list if (not file.startswith(".") and not re.search(r"\.([\w\d]){1,}$", file))]
                else:
                    filtered_dir_list = dir_list
                if len(filtered_dir_list) == 0:
                    cli.puts("!! There are no more directories to list")
                    exit() 
                filtered_dir_list.sort()
                for file in filtered_dir_list:
                    print (cli.puts_hide(">>> {}".format(filtered_dir_list.index(file) + 1)), '--', file)
                if (input_text):
                    option = input(input_text)
                else:
                    option = input("Choose an option: ")
                if not option.isnumeric():
                    cli.puts("!!! Invalid option selected")
                    exit()
                chosen = cli.cli_confirm("You have selected the directory '{}'. Keep this choice?".format(filtered_dir_list[int(option) - 1]))
                if chosen == 'y':
                    if not absolute_path:
                        return os.sep.join(options) + os.sep + filtered_dir_list[int(option) - 1]
                    else:
                        return dir + os.sep + os.sep.join(options) + os.sep + filtered_dir_list[int(option) - 1]

    def remote_file_chooser(self, c : Connection, dir = None, absolute_path = False, input_text = None, list_all = False):
        options = []
        dir = dir or c.cwd
        dir_list = c.sftp().listdir(dir)
        if not list_all:
            filtered_dir_list = [file for file in dir_list if (not file.startswith(".") and not re.search(r"\.([\w\d]){1,}$", file))]
        else:
            filtered_dir_list = dir_list
        filtered_dir_list.sort()
        for file in filtered_dir_list:
            print (cli.puts_hide(">>> {}".format(filtered_dir_list.index(file) + 1)), '--', file)
        if (input_text):
            option = input(input_text)
        else:
            option = input("Choose an option: ")
        if not option.isnumeric():
            cli.puts("!!! Invalid option selected")
            exit()
        while(option is not None):
            if (re.search(r"\.([\w\d]){1,}$", filtered_dir_list[int(option) - 1])):
                if not absolute_path:
                    return os.sep.join(options) + os.sep + filtered_dir_list[int(option) - 1]
                else:
                    return dir + os.sep + os.sep.join(options) + os.sep + filtered_dir_list[int(option) - 1]
            options.append(filtered_dir_list[int(option) - 1])
            c.sftp().chdir(dir + os.sep + os.sep.join(options))
            dir_list = c.sftp().listdir(dir + os.sep + os.sep.join(options))
            if not list_all:
                filtered_dir_list = [file for file in dir_list if (not file.startswith("."))]
            else:
                filtered_dir_list = dir_list
            filtered_dir_list.sort()
            for file in filtered_dir_list:
                print (cli.puts_hide(">>> {}".format(filtered_dir_list.index(file) + 1)), '--', file)
            if (input_text):
                option = input(input_text)
            else:
                option = input("Choose an option: ")
            if not option.isnumeric():
                cli.puts("!!! Invalid option selected")
                exit()

    def local_directory_chooser(self, dir = None, absolute_path = True, input_text = None, list_all = False):
        options = []
        if not dir: 
            return os.curdir
        dir = dir or os.curdir
        dir_list = os.listdir(dir)
        if not list_all:
            filtered_dir_list = [file for file in dir_list if (not file.startswith(".") and not re.search(r"\.([\w\d]){1,}$", file))]
        else:
            filtered_dir_list = dir_list
        filtered_dir_list.sort()
        for file in filtered_dir_list:
            print (cli.puts_hide(">>> {}".format(filtered_dir_list.index(file) + 1)), '--', file)
        if (input_text):
            option = input(input_text)
        else:
            option = input("Choose an option: ")
        if not option.isnumeric():
            cli.puts("!!! Invalid option selected")
            exit()
        chosen = cli.cli_confirm("You have selected the directory '{}'. Keep this choice?".format(filtered_dir_list[int(option) - 1]))
        if chosen == 'y':
            if not absolute_path:
                return os.sep.join(options) + filtered_dir_list[int(option) - 1]
            else:
                return dir + os.sep + os.sep.join(options) + filtered_dir_list[int(option) - 1]
        else:
            while(option is not None):
                options.append(filtered_dir_list[int(option) - 1])
                os.chdir(dir + os.sep + os.sep.join(options))
                dir_list = os.listdir(dir + os.sep + os.sep.join(options))
                if not list_all:
                    filtered_dir_list = [file for file in dir_list if (not file.startswith(".") and not re.search(r"\.([\w\d]){1,}$", file))]
                else:
                    filtered_dir_list = dir_list
                if len(filtered_dir_list) == 0:
                    cli.puts("!! There are no more directories to list")
                    exit() 
                filtered_dir_list.sort()
                for file in filtered_dir_list:
                    print (cli.puts_hide(">>> {}".format(filtered_dir_list.index(file) + 1)), '--', file)
                if (input_text):
                    option = input(input_text)
                else:
                    option = input("Choose an option: ")
                if not option.isnumeric():
                    cli.puts("!!! Invalid option selected")
                    exit()
                chosen = cli.cli_confirm("You have selected the directory '{}'. Keep this choice?".format(filtered_dir_list[int(option) - 1]))
                if chosen == 'y':
                    if not absolute_path:
                        return os.sep.join(options) + os.sep + filtered_dir_list[int(option) - 1]
                    else:
                        return dir + os.sep + os.sep.join(options) + os.sep + filtered_dir_list[int(option) - 1]

    def local_file_chooser(self, dir = None, absolute_path = False, input_text = None, list_all = False):
        options = []
        dir = dir or os.curdir
        dir_list = os.listdir(dir)
        if not list_all:
            filtered_dir_list = [file for file in dir_list if (not file.startswith(".") and not re.search(r"\.([\w\d]){1,}$", file))]
        else:
            filtered_dir_list = dir_list
        filtered_dir_list.sort()
        for file in filtered_dir_list:
            print (cli.puts_hide(">>> {}".format(filtered_dir_list.index(file) + 1)), '--', file)
        if (input_text):
            option = input(input_text)
        else:
            option = input("Choose an option: ")
        if not option.isnumeric():
            cli.puts("!!! Invalid option selected")
            exit()
        while(option is not None):
            if (re.search(r"\.([\w\d]){1,}$", filtered_dir_list[int(option) - 1])):
                if not absolute_path:
                    return os.sep.join(options) + os.sep + filtered_dir_list[int(option) - 1]
                else:
                    return dir + os.sep + os.sep.join(options) + os.sep + filtered_dir_list[int(option) - 1]
            options.append(filtered_dir_list[int(option) - 1])
            os.chdir(dir + os.sep + os.sep.join(options))
            dir_list = os.listdir(dir + os.sep + os.sep.join(options))
            if not list_all:
                filtered_dir_list = [file for file in dir_list if (not file.startswith("."))]
            else:
                filtered_dir_list = dir_list
            filtered_dir_list.sort()
            for file in filtered_dir_list:
                print (cli.puts_hide(">>> {}".format(filtered_dir_list.index(file) + 1)), '--', file)
            if (input_text):
                option = input(input_text)
            else:
                option = input("Choose an option: ")
            if not option.isnumeric():
                cli.puts("!!! Invalid option selected")
                exit()

    def _print(self) -> string:
        for key in self.options.keys():
            print (cli.puts_hide(">>> {}".format(key)), '--', self.options[key])

    def _get_option(self, option, confirm_prompt = True):
        if (option.strip() not in self.options.keys()):
                cli.puts("!!! Not a valid choice")
                exit()
        elif (self.options[option].lower() == 'exit'):
            cli.puts("!!! Exited")
            exit()
        elif (self.options[option].lower() == 'skip'):
            cli.puts("!!! Skipped")
            pass
        else:
            if confirm_prompt:
                choise = cli.cli_confirm("You have chosen '{}'. Continue?".format(self.options[option]))
                if (choise == 'y'):
                    return self.options[option]
                else:
                    cli.puts("!!! Aborted")
                    exit()
            return self.options[option]


import string

class CommandPrefix:
    def __init__(self, command, prefix) -> None:
        self.command = command
        self.prefix = prefix

    
    def prefix_command(self) -> string:
        if (self.prefix == ""):
            return "{}".format(self.command)
        return "{}'{}'".format(self.prefix, self.command)


from tif.fabric.logger import Logger
from tif.fabric import cli


class Warden:
    def __init__(self, environment_dir : str, services : dict) -> None:
        self.environment_dir = environment_dir
        self.logger = Logger()
        self.services = services

    def start(self, context):
        command = "warden svc start"
        self._log_command(command)
        context.run(command, pty=True)


    def stop(self, context):
        command = "warden svc stop"
        self._log_command(command)
        context.run(command, pty=True)

    def sign_certificate(self, context):
        site = cli.prompt(">>> Enter the site for the certificate: ").strip()
        if not site:
            self.logger.error('Site is required to sign a certificate')
            exit()
        with context.cd(self.environment_dir):
            command = "warden sign-certificate {}".format(site)
            self._log_command(command)
            context.run(command, pty=True)

    def shell(self, context):
        command = "warden shell"
        with context.cd(self.environment_dir):
            self._log_command(command)
            context.run(command, pty=True)

    def up(self, context):
        command = "warden env up"
        with context.cd(self.environment_dir):
            self._log_command(command)
            context.run(command, pty=True)

    def env_start(self, context):
        command = "warden env start"
        with context.cd(self.environment_dir):
            self._log_command(command)
            context.run(command, pty=True)

    def env_stop(self, context):
        command = "warden env stop"
        with context.cd(self.environment_dir):
            self._log_command(command)
            context.run(command, pty=True)

    def varnish_hist(self, context):
        command = "warden env exec {} varnishhist".format(self.services['varnish'])
        with context.cd(self.environment_dir):
            self._log_command(command)
            context.run(command, pty=True)

    def varnish_log(self, context):
        command = 'warden env exec -T {} varnishlog'.format(self.services['varnish'])
        with context.cd(self.environment_dir):
            self._log_command(command)
            context.run(command, pty=True)

    def varnish_reload(self, context):
        command = "warden env exec {} varnishreload".format(self.services['varnish'])
        with context.cd(self.environment_dir):
            self._log_command(command)
            context.run(command, pty=True)
    
    def varnish_purge(self, context):
        command = "warden env exec {} varnishadm 'ban req.url ~ .'".format(self.services['varnish'])
        with context.cd(self.environment_dir):
            self._log_command(command)
            context.run(command, pty=True)
            self.logger.success("Successfully purged data from varnish service")

    def debug(self, context):
        command = "warden debug"
        with context.cd(self.environment_dir):
            self._log_command(command)
            context.run(command, pty=True)
    
    def db_query(self, context, db_user, db_password, db, query):
        query = "mysql -u{} -p{}  {} -e \"{}\"".format(db_user, db_password, db, query)
        command = "warden env exec {} {}".format(self.services['db'], query)
        with context.cd(self.environment_dir):
            self._log_command(command)
            context.run(command, pty=True)

    def db_import(self, context):
        dump_file = cli.prompt(">>> Enter dump file to import. Must be a .sql file: ").strip()
        if dump_file:
            command = "cat {} | warden {} import".format(dump_file, self.services['db'])
            with context.cd(self.environment_dir):
                self._log_command(command)
                context.run(command, pty=True)
                self.logger.success("Dump file successfully imported!")
        else:
            self.logger.error("Aborted!")

    def bin_magento_exec(self, context, command):
        command = "warden env exec {} bin/magento {}".format(self.services['php'], command)
        with context.cd(self.environment_dir):
            self._log_command(command)
            context.run(command, pty=True)

    def artisan_exec(self, context, command):
        command = "warden env exec {} php artisan {}".format(self.services['php'], command)
        with context.cd(self.environment_dir):
            self._log_command(command)
            context.run(command, pty=True)

    def composer_install(self, context, flags = ''):
        command = "warden env exec {} composer install {}".format(self.services['php'], flags)
        with context.cd(self.environment_dir):
            self._log_command(command)
            context.run(command, pty=True)

    def composer_update(self, context, flags = ''):
        command = "warden env exec {} composer update {}".format(self.services['php'], flags)
        with context.cd(self.environment_dir):
            self._log_command(command)
            context.run(command, pty=True)
    
    def composer_run_setup_local_packages(self, context):
        command = "warden env exec {} composer run setup-local-packages".format(self.services['php'])
        with context.cd(self.environment_dir):
            self._log_command(command)
            context.run(command, pty=True)

    def composer_run_setup_vcs_packages(self, context):
        command = "warden env exec {} composer run setup-vcs-packages".format(self.services['php'])
        with context.cd(self.environment_dir):
            self._log_command(command)
            context.run(command, pty=True)
    
    def composer_require(self, context, flags = ''):
        package = cli.prompt(">>> Enter composer package to require: ").strip()
        if package:
            command = "warden env exec {} composer require {} {}".format(self.services['php'], package, flags)
            with context.cd(self.environment_dir):
                self._log_command(command)
                context.run(command, pty=True)

    def composer_require_dev(self, context, flags = ''):
        package = cli.prompt(">>> Enter dev composer package to require: ").strip()
        if package:
            command = "warden env exec {} composer require --dev {} {}".format(self.services['php'], package, flags)
            with context.cd(self.environment_dir):
                self._log_command(command)
                context.run(command, pty=True)

    def _log_command(self, command):
        self.logger.log("Running command '{}'".format(command))